@@ -17,11 +17,15 @@
  */
 
 #include "panel-platform.h"
+#include "applet-manager.h"
 #include "definitions.h"
+#include "toplevel.h"
 
 typedef struct
 {
 	ValaPanelCoreSettings *core_settings;
+	ValaPanelAppletManager *manager;
+	GHashTable *toplevels;
 } ValaPanelPlatformPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE(ValaPanelPlatform, vala_panel_platform, G_TYPE_OBJECT)
@@ -59,8 +63,8 @@ bool vala_panel_platform_init_settings_full(ValaPanelPlatform *self, const char
 {
 	ValaPanelPlatformPrivate *priv =
 	    (ValaPanelPlatformPrivate *)vala_panel_platform_get_instance_private(self);
-	priv->core_settings = vala_panel_core_settings_new(schema, path, backend);
-	return vala_panel_core_settings_init_unit_list(priv->core_settings);
+	priv->core_settings = vp_core_settings_new(schema, path, backend);
+	return vp_core_settings_init_unit_list(priv->core_settings);
 }
 
 bool vala_panel_platform_start_panels_from_profile(ValaPanelPlatform *self, GtkApplication *app,
@@ -73,6 +77,20 @@ bool vala_panel_platform_start_panels_from_profile(ValaPanelPlatform *self, GtkA
 	return false;
 }
 
+void vala_panel_platform_register_unit(ValaPanelPlatform *self, GtkWindow *unit)
+{
+	ValaPanelPlatformPrivate *priv =
+	    (ValaPanelPlatformPrivate *)vala_panel_platform_get_instance_private(self);
+	g_hash_table_add(priv->toplevels, unit);
+}
+
+void vala_panel_platform_unregister_unit(ValaPanelPlatform *self, GtkWindow *unit)
+{
+	ValaPanelPlatformPrivate *priv =
+	    (ValaPanelPlatformPrivate *)vala_panel_platform_get_instance_private(self);
+	g_hash_table_remove(priv->toplevels, unit);
+}
+
 ValaPanelCoreSettings *vala_panel_platform_get_settings(ValaPanelPlatform *self)
 {
 	ValaPanelPlatformPrivate *priv =
@@ -80,25 +98,18 @@ ValaPanelCoreSettings *vala_panel_platform_get_settings(ValaPanelPlatform *self)
 	return priv->core_settings;
 }
 
-void vala_panel_platform_init(ValaPanelPlatform *self)
+G_GNUC_INTERNAL ValaPanelAppletManager *vp_platform_get_manager(ValaPanelPlatform *self)
 {
 	ValaPanelPlatformPrivate *priv =
 	    (ValaPanelPlatformPrivate *)vala_panel_platform_get_instance_private(self);
-	priv->core_settings = NULL;
+	return priv->manager;
 }
 
-void vala_panel_platform_finalize(GObject *obj)
+bool vala_panel_platform_has_units_loaded(ValaPanelPlatform *self)
 {
-	ValaPanelPlatform *self = VALA_PANEL_PLATFORM(obj);
 	ValaPanelPlatformPrivate *priv =
 	    (ValaPanelPlatformPrivate *)vala_panel_platform_get_instance_private(self);
-	if (priv->core_settings)
-		vala_panel_core_settings_free(priv->core_settings);
-}
-
-void vala_panel_platform_class_init(ValaPanelPlatformClass *klass)
-{
-	G_OBJECT_CLASS(klass)->finalize = vala_panel_platform_finalize;
+	return g_hash_table_size(priv->toplevels);
 }
 
 bool vala_panel_platform_edge_available(ValaPanelPlatform *self, GtkWindow *top,
@@ -111,3 +122,32 @@ bool vala_panel_platform_edge_available(ValaPanelPlatform *self, GtkWindow *top,
 		                                                           monitor);
 	return false;
 }
+
+static void vala_panel_platform_init(ValaPanelPlatform *self)
+{
+	ValaPanelPlatformPrivate *priv =
+	    (ValaPanelPlatformPrivate *)vala_panel_platform_get_instance_private(self);
+	priv->core_settings = NULL;
+	priv->manager       = vp_applet_manager_new();
+	priv->toplevels     = g_hash_table_new_full(g_direct_hash,
+                                                g_direct_equal,
+                                                (GDestroyNotify)gtk_widget_destroy,
+                                                NULL);
+}
+
+static void vala_panel_platform_finalize(GObject *obj)
+{
+	ValaPanelPlatform *self = VALA_PANEL_PLATFORM(obj);
+	ValaPanelPlatformPrivate *priv =
+	    (ValaPanelPlatformPrivate *)vala_panel_platform_get_instance_private(self);
+	g_hash_table_unref(priv->toplevels);
+	if (priv->core_settings)
+		vp_core_settings_free(priv->core_settings);
+	g_clear_object(&priv->manager);
+	G_OBJECT_CLASS(vala_panel_platform_parent_class)->finalize(obj);
+}
+
+static void vala_panel_platform_class_init(ValaPanelPlatformClass *klass)
+{
+	G_OBJECT_CLASS(klass)->finalize = vala_panel_platform_finalize;
+}
